@@ -8,7 +8,7 @@ test_that("k-way matching with one odd-ball per-group", {
     Xs <- cbind(c(-4, runif(198), 5), c(-4, runif(198), 5), runif(200))
     
     retained.ids <- suppressWarnings(cb.align.kway_match(Ts, data.frame(Covar=Xs), match.form = "Covar.1 + Covar.2 + Covar.3",
-                                                         match.args=list(method="nearest", caliper=0.3, exact=NULL,replace=FALSE)))
+                                                         match.args=list(method="nearest", caliper=0.3, exact=NULL,replace=FALSE))$Retained.Ids)
     
     excl_samps.s1 <- !(1 %in% retained.ids)
     excl_samps.s200 <- !(200 %in% retained.ids)
@@ -28,15 +28,15 @@ test_that("as unbalancedness increases, fewer samples retained by k-way matching
     
     sim.high <- cb.sims.sim_sigmoid(n=200, unbalancedness=1)
     retained.high <- cb.align.kway_match(sim.high$Ts, data.frame(Covar=sim.high$Xs),
-                                         match.form="Covar")
+                                         match.form="Covar")$Retained.Ids
     
     sim.mod <- cb.sims.sim_sigmoid(n=200, unbalancedness=1.5)
     retained.mod <- cb.align.kway_match(sim.mod$Ts, data.frame(Covar=sim.mod$Xs),
-                                        match.form="Covar")
+                                        match.form="Covar")$Retained.Ids
     
     sim.low <- cb.sims.sim_sigmoid(n=200, unbalancedness=2.5)
     retained.low <- cb.align.kway_match(sim.low$Ts, data.frame(Covar=sim.low$Xs),
-                                        match.form="Covar", retain.ratio=0)
+                                        match.form="Covar", retain.ratio=0)$Retained.Ids
     
     rank.lengths <- rank(c(length(retained.high), length(retained.mod), length(retained.low)))
     return(all(rank.lengths == c(3, 2, 1)))
@@ -48,7 +48,7 @@ test_that("as unbalancedness increases, fewer samples retained by k-way matching
 test_that("K-way matching throws warning when samples retained is low", {
   sim.low <- cb.sims.sim_sigmoid(n=200, unbalancedness=2)
   expect_warning(cb.align.kway_match(sim.low$Ts, data.frame(Covar=sim.low$Xs),
-                                     match.form="Covar", retain.ratio = 0.7))
+                                     match.form="Covar", retain.ratio = 0.8))
 })
 
 test_that("K-way matching throws error when no samples retained", {
@@ -68,7 +68,7 @@ approx.overlap <- function(X1, X2, nbreaks=100) {
 test_that("K-way matching increases covariate overlap", {
   sim.mod <- cb.sims.sim_sigmoid(n=200, unbalancedness = 2)
   retained.ids <- cb.align.kway_match(sim.mod$Ts, data.frame(Covar=sim.mod$Xs),
-                                      match.form="Covar", retain.ratio = 0)
+                                      match.form="Covar", retain.ratio = 0)$Retained.Ids
   
   Ts.tilde <- sim.mod$Ts[retained.ids]
   Xs.tilde <- sim.mod$Xs[retained.ids]
